using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using SciChart.Charting;
using SciChart.Charting.ChartModifiers;
using SciChart.Charting.Common.Helpers;
using SciChart.Charting.Model.ChartSeries;
using SciChart.Charting.Model.DataSeries;
using SciChart.Charting.Visuals.Annotations;
using SciChart.Charting.Visuals.Axes;
using SciChart.Charting.Visuals.PointMarkers;
using SciChart.Data.Model;
using SciChart.Examples.ExternalDependencies.Common;
using SciChart.Examples.ExternalDependencies.Controls.Toolbar2D.Converters;
using SciChart_AxisBindingandAnnotationBinding;

namespace SciChart.Examples.Examples.UseSciChartWithMvvm.SciChartMVVMBinding
{
    public class SciChartMvvmBindingsViewModel : BaseViewModel
    {
        private Type _annotationType;

        public SciChartMvvmBindingsViewModel()
        {
            AnnotationViewModels = new ObservableCollection<IAnnotationViewModel>();
            InitializeAnnotations();

            RenderableSeries = new ObservableCollection<IRenderableSeriesViewModel>();
            InitializeRenderableSeries();

            XAxes = new ObservableCollection<IAxisViewModel>();
            YAxes = new ObservableCollection<IAxisViewModel>();

            InitializeAxes();
        }

        public Type AnnotationType
        {
            get => _annotationType;
            set
            {
                _annotationType = value;
                OnPropertyChanged("AnnotationType");
            }
        }

        public ICommand AnnotationCreatedCommand => new ActionCommand<AnnotationCreationMVVMArgs>(e =>
        {
            var annotation = e.NewAnnotationViewModel;

            if (annotation != null && annotation is MyCompositeAnnotationViewModel myCA)
            {
                myCA.Annotations.Add(new LineAnnotationViewModel
                {
                    X1 = 0,
                    X2 = 1,
                    Y1 = 0.5,
                    Y2 = 0.5,
                    CoordinateMode = AnnotationCoordinateMode.Relative,
                    Stroke = Colors.Red,
                });

                annotation.IsEditable = true;
                annotation.CanEditText = true;
                annotation.IsSelected = true;
            }
        });

        private void InitializeAnnotations()
        {
            var myAnnVM = new MyCompositeAnnotationViewModel()
            {
                X1 = 1,
                X2 = 8,
                Y1 = 2,
                Y2 = 4,
                CoordinateMode = AnnotationCoordinateMode.Absolute,
                DragDirections = XyDirection.XYDirection,
                ResizeDirections = XyDirection.XYDirection,
            };
            myAnnVM.Annotations.Add(new LineAnnotationViewModel
            {
                X1 = 0,
                X2 = 1,
                Y1 = 0.5,
                Y2 = 0.5,
                CoordinateMode = AnnotationCoordinateMode.Relative,
                Stroke = Colors.Red,
            });

            AnnotationViewModels.Add(myAnnVM);
        }

        private void InitializeRenderableSeries()
        {
            var renderableSeries = new LineRenderableSeriesViewModel
            {
                Stroke = Colors.Blue,
                StrokeThickness = 2,
                PointMarker = new EllipsePointMarker()
            };

            var dataSeries = new UniformXyDataSeries<double>();

            renderableSeries.DataSeries = dataSeries;

            for (int i = 0; i < 11; ++i)
            {
                dataSeries.Append(i - 3);
            }

            RenderableSeries.Add(renderableSeries);
        }

        private void InitializeAxes()
        {
            var xNumAxis = new NumericAxisViewModel
            {
                AxisAlignment = AxisAlignment.Bottom,
                AxisTitle = "XAxis",
                DrawMajorBands = false,
                TextFormatting = "0.00#",
                VisibleRange = new DoubleRange(0, 10),
                BorderThickness = new Thickness(3),
                BorderBrush = new SolidColorBrush(Colors.CadetBlue)
            };

            XAxes.Add(xNumAxis);

            var xDateTimeAxis = new DateTimeAxisViewModel
            {
                AxisAlignment = AxisAlignment.Top,
                Id = "DateTimeAxis",
                VisibleRange = new DateRange(new DateTime(2017, 1, 1), new DateTime(2017, 1, 31)),

                // Apply a style declared in the View
                StyleKey = "DateTimeAxisStyle"
            };

            XAxes.Add(xDateTimeAxis);

            var yNumAxis = new NumericAxisViewModel
            {
                AxisTitle = "YAxis",
                DrawMajorBands = false,
                TextFormatting = "0.0#",
                VisibleRange = new DoubleRange(-10, 10)
            };

            YAxes.Add(yNumAxis);

            var yTimeSpanAxis = new TimeSpanAxisViewModel
            {
                Id = "TimeSpanAxis",
                DrawMajorBands = false,
                VisibleRange = new TimeSpanRange(TimeSpan.FromHours(1), TimeSpan.FromHours(24)),
                AxisAlignment = AxisAlignment.Left
            };

            YAxes.Add(yTimeSpanAxis);
        }

        public ObservableCollection<IAnnotationViewModel> AnnotationViewModels { get; set; }
        public ObservableCollection<IRenderableSeriesViewModel> RenderableSeries { get; set; }

        public ObservableCollection<IAxisViewModel> XAxes { get; set; }
        public ObservableCollection<IAxisViewModel> YAxes { get; set; }
    }
}